@@ -37,6 +37,7 @@
 #include "stl_util.h"
 
 #include "libkleo_debug.h"
+#include "utils/formatting.h"
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -69,8 +70,8 @@ public:
     explicit Model(KeyFilterManager::Private *p)
         : QAbstractListModel(nullptr), m_keyFilterManagerPrivate(p) {}
 
-    int rowCount(const QModelIndex &) const Q_DECL_OVERRIDE;
-    QVariant data(const QModelIndex &idx, int role) const Q_DECL_OVERRIDE;
+    int rowCount(const QModelIndex &) const override;
+    QVariant data(const QModelIndex &idx, int role) const override;
     /* upgrade to public */ using QAbstractListModel::reset;
 };
 
@@ -111,7 +112,7 @@ public:
     {
         setRevoked(NotSet);
         setValidity(IsAtLeast);
-        setValidityReferenceLevel(UserID::Marginal); // Full?
+        setValidityReferenceLevel(UserID::Marginal);
         setSpecificity(UINT_MAX - 2); // overly high for ordering
 
         setName(i18n("Trusted Certificates"));
@@ -120,6 +121,23 @@ public:
     }
 };
 
+class FullCertificatesKeyFilter : public DefaultKeyFilter
+{
+public:
+    FullCertificatesKeyFilter()
+        : DefaultKeyFilter()
+    {
+        setRevoked(NotSet);
+        setValidity(IsAtLeast);
+        setValidityReferenceLevel(UserID::Full);
+        setSpecificity(UINT_MAX - 3);
+
+        setName(i18n("Fully Trusted Certificates"));
+        setId(QStringLiteral("full-certificates"));
+        setMatchContexts(Filtering);
+    }
+};
+
 class OtherCertificatesKeyFilter : public DefaultKeyFilter
 {
 public:
@@ -129,37 +147,126 @@ public:
         setHasSecret(NotSet);
         setValidity(IsAtMost);
         setValidityReferenceLevel(UserID::Never);
-        setSpecificity(UINT_MAX - 3); // overly high for ordering
+        setSpecificity(UINT_MAX - 4); // overly high for ordering
 
         setName(i18n("Other Certificates"));
         setId(QStringLiteral("other-certificates"));
         setMatchContexts(Filtering);
     }
 };
+
+/* This filter selects only VS-compliant keys if Kleopatra is used in
+ * CO_DE_VS mode.  */
+class DeVsCompliantKeyFilter : public DefaultKeyFilter
+{
+public:
+    DeVsCompliantKeyFilter()
+        : DefaultKeyFilter()
+    {
+        setName(i18n("VS-compliant Certificates"));
+        setId(QStringLiteral("vs-compliant-certificates"));
+        setSpecificity(UINT_MAX - 5); // overly high for ordering
+    }
+    bool matches (const Key &key, MatchContexts contexts) const override
+    {
+        return (contexts & Filtering) && Formatting::isKeyDeVs(key);
+    }
+};
+
+/* This filter selects only invalid keys (i.e. those where not all
+ * UIDs are at least fully valid).  */
+class KeyNotValidFilter : public DefaultKeyFilter
+{
+public:
+    KeyNotValidFilter()
+        : DefaultKeyFilter()
+    {
+        setName(i18n("Not validated Certificates"));
+        setId(QStringLiteral("not-validated-certificates"));
+        setSpecificity(UINT_MAX - 6); // overly high for ordering
+    }
+    bool matches (const Key &key, MatchContexts contexts) const override
+    {
+        return (contexts & Filtering) && !Formatting::uidsHaveFullValidity(key);
+    }
+};
+
+/* This filter gives valid keys (i.e. those where all UIDs are at
+ * least fully valid) a light green background if Kleopatra is used in
+ * CO_DE_VS mode.  */
+class KeyValidAppearanceFilter : public DefaultKeyFilter
+{
+public:
+    KeyValidAppearanceFilter()
+        : DefaultKeyFilter()
+    {
+        setBgColor(QColor("lightgreen"));
+    }
+    bool matches (const Key &key, MatchContexts contexts) const override
+    {
+        return (contexts & Appearance) && Formatting::uidsHaveFullValidity(key);
+    }
+};
+
+/* This filter gives invalid keys (i.e. those where not all UIDs are
+ * at least fully valid) a light red background if Kleopatra is used
+ * in CO_DE_VS mode.  */
+class KeyNotValidAppearanceFilter : public DefaultKeyFilter
+{
+public:
+    KeyNotValidAppearanceFilter()
+        : DefaultKeyFilter()
+    {
+        setBgColor(QColor("lightpink"));
+    }
+    bool matches (const Key &key, MatchContexts contexts) const override
+    {
+        return (contexts & Appearance) && !Formatting::uidsHaveFullValidity(key);
+    }
+};
+
 }
 
 static std::vector<std::shared_ptr<KeyFilter>> defaultFilters()
 {
     std::vector<std::shared_ptr<KeyFilter> > result;
-    result.reserve(3);
+    result.reserve(6);
     result.push_back(std::shared_ptr<KeyFilter>(new MyCertificatesKeyFilter));
     result.push_back(std::shared_ptr<KeyFilter>(new TrustedCertificatesKeyFilter));
+    result.push_back(std::shared_ptr<KeyFilter>(new FullCertificatesKeyFilter));
     result.push_back(std::shared_ptr<KeyFilter>(new OtherCertificatesKeyFilter));
     result.push_back(std::shared_ptr<KeyFilter>(new AllCertificatesKeyFilter));
+    result.push_back(std::shared_ptr<KeyFilter>(new KeyNotValidFilter));
+    if (Formatting::complianceMode() == QStringLiteral("de-vs")) {
+        result.push_back(std::shared_ptr<KeyFilter>(new DeVsCompliantKeyFilter));
+    }
+    return result;
+}
+
+static std::vector<std::shared_ptr<KeyFilter>> defaultAppearanceFilters()
+{
+    std::vector<std::shared_ptr<KeyFilter> > result;
+    if (Formatting::complianceMode() == QStringLiteral("de-vs")) {
+        result.reserve(2);
+        result.push_back(std::shared_ptr<KeyFilter>(new KeyValidAppearanceFilter));
+        result.push_back(std::shared_ptr<KeyFilter>(new KeyNotValidAppearanceFilter));
+    }
     return result;
 }
 
 class KeyFilterManager::Private
 {
 public:
-    Private() : filters(), model(this) {}
+    Private() : filters(), appearanceFilters(), model(this) {}
     void clear()
     {
         filters.clear();
+        appearanceFilters.clear();
         model.reset();
     }
 
     std::vector<std::shared_ptr<KeyFilter>> filters;
+    std::vector<std::shared_ptr<KeyFilter>> appearanceFilters;
     Model model;
 };
 
@@ -233,6 +340,7 @@ void KeyFilterManager::reload()
     d->clear();
 
     d->filters = defaultFilters();
+    d->appearanceFilters = defaultAppearanceFilters();
     KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("libkleopatrarc"));
 
     const QStringList groups = config->groupList().filter(QRegularExpression(QStringLiteral("^Key Filter #\\d+$")));
@@ -293,30 +401,52 @@ int Model::rowCount(const QModelIndex &) const
 
 QVariant Model::data(const QModelIndex &idx, int role) const
 {
-    if ((role != Qt::DisplayRole && role != Qt::EditRole &&
-            role != Qt::ToolTipRole && role != Qt::DecorationRole) ||
-            !idx.isValid() || idx.model() != this ||
-            idx.row() < 0 || static_cast<unsigned>(idx.row()) >  m_keyFilterManagerPrivate->filters.size()) {
+    if (!idx.isValid()
+        || idx.model() != this
+        || idx.row() < 0
+        || static_cast<unsigned>(idx.row()) >  m_keyFilterManagerPrivate->filters.size()) {
         return QVariant();
     }
-    if (role == Qt::DecorationRole) {
-        return m_keyFilterManagerPrivate->filters[idx.row()]->icon();
-    } else {
-        return m_keyFilterManagerPrivate->filters[idx.row()]->name();
+
+    const auto filter = m_keyFilterManagerPrivate->filters[idx.row()];
+    switch (role) {
+    case Qt::DecorationRole:
+        return filter->icon();
+
+    case Qt::DisplayRole:
+    case Qt::EditRole:
+    case Qt::ToolTipRole:  /* Most useless tooltip ever.  */
+        return filter->name();
+
+    case Qt::UserRole:
+        return filter->id();
+
+    default:
+        return QVariant();
     }
 }
 
+static KeyFilter::FontDescription get_fontdescription(const std::vector<std::shared_ptr<KeyFilter>> &filters, const Key &key, const KeyFilter::FontDescription &initial)
+{
+    return kdtools::accumulate_if(filters.begin(), filters.end(),
+                                  [&key](const std::shared_ptr<KeyFilter> &filter) {
+                                      return filter->matches(key, KeyFilter::Appearance);
+                                  },
+                                  initial,
+                                  [](const KeyFilter::FontDescription &lhs,
+                                     const std::shared_ptr<KeyFilter> &rhs) {
+                                      return lhs.resolve(rhs->fontDescription());
+                                  });
+}
+
 QFont KeyFilterManager::font(const Key &key, const QFont &baseFont) const
 {
-    return kdtools::accumulate_if(d->filters.begin(), d->filters.end(),
-                                  [&key](const std::shared_ptr<KeyFilter> &filter) {
-                                      return filter->matches(key, KeyFilter::Appearance);
-                                  },
-                                  KeyFilter::FontDescription(),
-                                  [](const KeyFilter::FontDescription &lhs,
-                                     const std::shared_ptr<KeyFilter> &rhs) {
-                                      return lhs.resolve(rhs->fontDescription());
-                                  }).font(baseFont);
+    KeyFilter::FontDescription fd;
+
+    fd = get_fontdescription(d->appearanceFilters, key, KeyFilter::FontDescription());
+    fd = get_fontdescription(d->filters, key, fd);
+
+    return fd.font(baseFont);
 }
 
 static QColor get_color(const std::vector<std::shared_ptr<KeyFilter>> &filters, const Key &key, QColor(KeyFilter::*fun)() const)
@@ -324,7 +454,7 @@ static QColor get_color(const std::vector<std::shared_ptr<KeyFilter>> &filters,
     const auto it = std::find_if(filters.cbegin(), filters.cend(),
                                  [&fun, &key](const std::shared_ptr<KeyFilter> &filter) {
                                     return filter->matches(key, KeyFilter::Appearance)
-                                    && !(filter.get()->*fun)().isValid();
+                                    && (filter.get()->*fun)().isValid();
                                  });
     if (it == filters.cend()) {
         return QColor();
@@ -349,17 +479,34 @@ static QString get_string(const std::vector<std::shared_ptr<KeyFilter>> &filters
 
 QColor KeyFilterManager::bgColor(const Key &key) const
 {
-    return get_color(d->filters, key, &KeyFilter::bgColor);
+    QColor color;
+
+    color = get_color(d->appearanceFilters, key, &KeyFilter::bgColor);
+    if (!color.isValid()) {
+        color = get_color(d->filters, key, &KeyFilter::bgColor);
+    }
+    return color;
 }
 
 QColor KeyFilterManager::fgColor(const Key &key) const
 {
-    return get_color(d->filters, key, &KeyFilter::fgColor);
+    QColor color;
+
+    color = get_color(d->appearanceFilters, key, &KeyFilter::fgColor);
+    if (!color.isValid()) {
+        color = get_color(d->filters, key, &KeyFilter::fgColor);
+    }
+    return color;
 }
 
 QIcon KeyFilterManager::icon(const Key &key) const
 {
-    const QString icon = get_string(d->filters, key, &KeyFilter::icon);
+    QString icon;
+
+    icon = get_string(d->appearanceFilters, key, &KeyFilter::icon);
+    if (icon.isEmpty()) {
+        icon = get_string(d->filters, key, &KeyFilter::icon);
+    }
     return icon.isEmpty() ? QIcon() : QIcon::fromTheme(icon);
 }
 
