@@ -237,7 +237,7 @@ KeyCache::~KeyCache() {}
 
 void KeyCache::enableFileSystemWatcher(bool enable)
 {
-    Q_FOREACH (const auto &i, d->m_fsWatchers) {
+    for (const auto &i : qAsConst(d->m_fsWatchers)) {
         i->setEnabled(enable);
     }
 }
@@ -928,7 +928,7 @@ void KeyCache::insert(const std::vector<Key> &keys)
     by_subkeyid.swap(d->by.subkeyid);
     by_chainid.swap(d->by.chainid);
 
-    Q_FOREACH (const Key &key, sorted) {
+    for (const Key &key : qAsConst(sorted)) {
         Q_EMIT added(key);
     }
 
