@@ -69,79 +69,6 @@ using namespace Kleo;
 
 Q_DECLARE_METATYPE(GpgME::Key)
 
-/****************************************************************************
-**
-** Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: Qt Software Information (qt-info@nokia.com)
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
-**
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License versions 2.0 or 3.0 as published by the Free
-** Software Foundation and appearing in the file LICENSE.GPL included in
-** the packaging of this file.  Please review the following information
-** to ensure GNU General Public Licensing requirements will be met:
-** http://www.fsf.org/licensing/licenses/info/GPLv2.html and
-** http://www.gnu.org/copyleft/gpl.html.  In addition, as a special
-** exception, Nokia gives you certain additional rights. These rights
-** are described in the Nokia Qt GPL Exception version 1.3, included in
-** the file GPL_EXCEPTION.txt in this package.
-**
-** Qt for Windows(R) Licensees
-** As a special exception, Nokia, as the sole copyright holder for Qt
-** Designer, grants users of the Qt/Eclipse Integration plug-in the
-** right for the Qt/Eclipse Integration to link to functionality
-** provided by Qt Designer and its related libraries.
-**
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at qt-sales@nokia.com.
-**
-****************************************************************************/
-
-/*
-    These functions are based on Peter J. Weinberger's hash function
-    (from the Dragon Book). The constant 24 in the original function
-    was replaced with 23 to produce fewer collisions on input such as
-    "a", "aa", "aaa", "aaaa", ...
-*/
-
-// adjustment to null-terminated strings
-// (c) 2008 Klarälvdalens Datakonsult AB
-static uint hash(const uchar *p)
-{
-    uint h = 0;
-    uint g;
-
-    while (*p) {
-        h = (h << 4) + *p++;
-        if ((g = (h & 0xf0000000)) != 0) {
-            h ^= g >> 23;
-        }
-        h &= ~g;
-    }
-    return h;
-}
-
-//
-// end Nokia-copyrighted code
-//
-
-static inline uint qHash(const char *data)
-{
-    if (!data) {
-        return 1;    // something != 0
-    }
-    return ::hash(reinterpret_cast<const uchar *>(data));
-}
-
 class AbstractKeyListModel::Private
 {
 public:
@@ -268,6 +195,7 @@ QVariant AbstractKeyListModel::headerData(int section, Qt::Orientation o, int ro
             switch (section) {
             case PrettyName:       return i18n("Name");
             case PrettyEMail:      return i18n("E-Mail");
+            case Validity:         return i18n("Validity");
             case ValidFrom:        return i18n("Valid From");
             case ValidUntil:       return i18n("Valid Until");
             case TechnicalDetails: return i18n("Details");
@@ -319,6 +247,8 @@ QVariant AbstractKeyListModel::data(const QModelIndex &index, int role) const
             } else {
                 return QVariant();
             }
+	case Validity:
+	    return Formatting::complianceStringShort(key);
         case ValidFrom:
             if (role == Qt::EditRole) {
                 return Formatting::creationDate(key);
@@ -368,17 +298,17 @@ public:
     ~TableModelMixin() {}
 
     using Base::index;
-    QModelIndex index(int row, int column, const QModelIndex &pidx = QModelIndex()) const Q_DECL_OVERRIDE
+    QModelIndex index(int row, int column, const QModelIndex &pidx = QModelIndex()) const override
     {
         return this->hasIndex(row, column, pidx) ? this->createIndex(row, column, nullptr) : QModelIndex();
     }
 
 private:
-    QModelIndex parent(const QModelIndex &) const Q_DECL_OVERRIDE
+    QModelIndex parent(const QModelIndex &) const override
     {
         return QModelIndex();
     }
-    bool hasChildren(const QModelIndex &pidx) const Q_DECL_OVERRIDE
+    bool hasChildren(const QModelIndex &pidx) const override
     {
         return (pidx.model() == this || !pidx.isValid()) && this->rowCount(pidx) > 0 && this->columnCount(pidx) > 0;
     }
@@ -396,17 +326,17 @@ public:
     explicit FlatKeyListModel(QObject *parent = nullptr);
     ~FlatKeyListModel();
 
-    int rowCount(const QModelIndex &pidx) const Q_DECL_OVERRIDE
+    int rowCount(const QModelIndex &pidx) const override
     {
         return pidx.isValid() ? 0 : mKeysByFingerprint.size();
     }
 
 private:
-    Key doMapToKey(const QModelIndex &index) const Q_DECL_OVERRIDE;
-    QModelIndex doMapFromKey(const Key &key, int col) const Q_DECL_OVERRIDE;
-    QList<QModelIndex> doAddKeys(const std::vector<Key> &keys) Q_DECL_OVERRIDE;
-    void doRemoveKey(const Key &key) Q_DECL_OVERRIDE;
-    void doClear() Q_DECL_OVERRIDE {
+    Key doMapToKey(const QModelIndex &index) const override;
+    QModelIndex doMapFromKey(const Key &key, int col) const override;
+    QList<QModelIndex> doAddKeys(const std::vector<Key> &keys) override;
+    void doRemoveKey(const Key &key) override;
+    void doClear() override {
         mKeysByFingerprint.clear();
     }
 
@@ -421,22 +351,22 @@ public:
     explicit HierarchicalKeyListModel(QObject *parent = nullptr);
     ~HierarchicalKeyListModel();
 
-    int rowCount(const QModelIndex &pidx) const Q_DECL_OVERRIDE;
+    int rowCount(const QModelIndex &pidx) const override;
     using AbstractKeyListModel::index;
-    QModelIndex index(int row, int col, const QModelIndex &pidx) const Q_DECL_OVERRIDE;
-    QModelIndex parent(const QModelIndex &idx) const Q_DECL_OVERRIDE;
+    QModelIndex index(int row, int col, const QModelIndex &pidx) const override;
+    QModelIndex parent(const QModelIndex &idx) const override;
 
-    bool hasChildren(const QModelIndex &pidx) const Q_DECL_OVERRIDE
+    bool hasChildren(const QModelIndex &pidx) const override
     {
         return rowCount(pidx) > 0;
     }
 
 private:
-    Key doMapToKey(const QModelIndex &index) const Q_DECL_OVERRIDE;
-    QModelIndex doMapFromKey(const Key &key, int col) const Q_DECL_OVERRIDE;
-    QList<QModelIndex> doAddKeys(const std::vector<Key> &keys) Q_DECL_OVERRIDE;
-    void doRemoveKey(const Key &key) Q_DECL_OVERRIDE;
-    void doClear() Q_DECL_OVERRIDE {
+    Key doMapToKey(const QModelIndex &index) const override;
+    QModelIndex doMapFromKey(const Key &key, int col) const override;
+    QList<QModelIndex> doAddKeys(const std::vector<Key> &keys) override;
+    void doRemoveKey(const Key &key) override;
+    void doClear() override {
         mTopLevels.clear();
         mKeysByFingerprint.clear();
         mKeysByExistingParent.clear();
@@ -776,7 +706,7 @@ static std::vector<Key> topological_sort(const std::vector<Key> &keys)
 
     std::vector<Key> result;
     result.reserve(keys.size());
-    Q_FOREACH (int i, order) {
+    for (int i : qAsConst(order)) {
         result.push_back(keys[i]);
     }
     return result;
@@ -824,7 +754,7 @@ QList<QModelIndex> HierarchicalKeyListModel::doAddKeys(const std::vector<Key> &k
             std::vector<Key>::iterator last = mTopLevels.begin();
             std::vector<Key>::iterator lastFP = mKeysByFingerprint.begin();
 
-            Q_FOREACH (const Key &k, children) {
+            for (const Key &k : qAsConst(children)) {
                 last = qBinaryFind(last, mTopLevels.end(), k, _detail::ByFingerprint<std::less>());
                 Q_ASSERT(last != mTopLevels.end());
                 const int row = std::distance(mTopLevels.begin(), last);
@@ -877,7 +807,7 @@ QList<QModelIndex> HierarchicalKeyListModel::doAddKeys(const std::vector<Key> &k
     }
     //Q_EMIT dataChanged for all parents with new children. This triggers KeyListSortFilterProxyModel to
     //show a parent node if it just got children matching the proxy's filter
-    Q_FOREACH (const Key &i, changedParents) {
+    for (const Key &i : qAsConst(changedParents)) {
         const QModelIndex idx = index(i);
         if (idx.isValid()) {
             Q_EMIT dataChanged(idx.sibling(idx.row(), 0), idx.sibling(idx.row(), NumColumns - 1));
