@@ -102,7 +102,7 @@ Kleo::KeyListView::KeyListView(const ColumnStrategy *columnStrategy, const Displ
     for (int col = 0; !columnStrategy->title(col).isEmpty(); ++col) {
         headerItem()->setText(col, columnStrategy->title(col));
         header()->resizeSection(col, columnStrategy->width(col, fm));
-        header()->setResizeMode(col, columnStrategy->resizeMode(col));
+        header()->setSectionResizeMode(col, columnStrategy->resizeMode(col));
     }
 
     setAllColumnsShowFocus(true);
@@ -540,7 +540,7 @@ Kleo::KeyListViewItem *Kleo::KeyListView::firstChild() const
 Kleo::KeyListViewItem *Kleo::KeyListView::selectedItem() const
 {
     QList<KeyListViewItem *> selection = selectedItems();
-    if (selection.size() == 0) {
+    if (selection.isEmpty()) {
         return nullptr;
     }
     return selection.first();
